/*
 * Copyright (c) 1994, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package glue.java.lang;

import blues.Jail;
import blues.Log;
import impl.java.lang.ClassLoaderPeer;
import impl.java.lang.Reflection;
import impl.sun.security.util.SecurityConstants;

public class Class {

    /**
     * Returns the {@code Class} object associated with the class or interface
     * with the given string name. Invoking this method is equivalent to:
     *
     * <blockquote> {@code Class.forName(className, true, currentLoader)}
     * </blockquote>
     *
     * where {@code currentLoader} denotes the defining class loader of the
     * current class.
     *
     * <p>
     * For example, the following code fragment returns the runtime
     * {@code Class} descriptor for the class named {@code java.lang.Thread}:
     *
     * <blockquote> {@code Class t = Class.forName("java.lang.Thread")}
     * </blockquote>
     * <p>
     * A call to {@code forName("X")} causes the class named {@code X} to be
     * initialized.
     *
     * @param className
     *            the fully qualified name of the desired class.
     * @return the {@code Class} object for the class with the specified name.
     * @exception LinkageError
     *                if the linkage fails
     * @exception ExceptionInInitializerError
     *                if the initialization provoked by this method fails
     * @exception ClassNotFoundException
     *                if the class cannot be located
     */
    public static java.lang.Class forName(String className) throws ClassNotFoundException {
        java.lang.Class caller = Reflection.getCallerClass();
        return forName0(className, true, ClassLoaderPeer.getClassLoader(caller), caller);
    }

    /**
     * Returns the {@code Class} object associated with the class or interface
     * with the given string name, using the given class loader. Given the fully
     * qualified name for a class or interface (in the same format returned by
     * {@code getName}) this method attempts to locate, load, and link the class
     * or interface. The specified class loader is used to load the class or
     * interface. If the parameter {@code loader} is null, the class is loaded
     * through the bootstrap class loader. The class is initialized only if the
     * {@code initialize} parameter is {@code true} and if it has not been
     * initialized earlier.
     *
     * <p>
     * If {@code name} denotes a primitive type or void, an attempt will be made
     * to locate a user-defined class in the unnamed package whose name is
     * {@code name}. Therefore, this method cannot be used to obtain any of the
     * {@code Class} objects representing primitive types or void.
     *
     * <p>
     * If {@code name} denotes an array class, the component type of the array
     * class is loaded but not initialized.
     *
     * <p>
     * For example, in an instance method the expression:
     *
     * <blockquote> {@code Class.forName("Foo")} </blockquote>
     *
     * is equivalent to:
     *
     * <blockquote>
     * {@code Class.forName("Foo", true, this.getClass().getClassLoader())}
     * </blockquote>
     *
     * Note that this method throws errors related to loading, linking or
     * initializing as specified in Sections 12.2, 12.3 and 12.4 of <em>The
     * Java Language Specification</em>. Note that this method does not check
     * whether the requested class is accessible to its caller.
     *
     * <p>
     * If the {@code loader} is {@code null}, and a security manager is present,
     * and the caller's class loader is not null, then this method calls the
     * security manager's {@code checkPermission} method with a
     * {@code RuntimePermission("getClassLoader")} permission to ensure it's ok
     * to access the bootstrap class loader.
     *
     * @param name
     *            fully qualified name of the desired class
     * @param initialize
     *            whether the class must be initialized
     * @param loader
     *            class loader from which the class must be loaded
     * @return class object representing the desired class
     *
     * @exception LinkageError
     *                if the linkage fails
     * @exception ExceptionInInitializerError
     *                if the initialization provoked by this method fails
     * @exception ClassNotFoundException
     *                if the class cannot be located by the specified class
     *                loader
     *
     * @see java.lang.Class#forName(String)
     * @see java.lang.ClassLoader
     * @since 1.2
     */
    public static java.lang.Class forName(String name, boolean initialize, ClassLoader loader)
            throws ClassNotFoundException {
        java.lang.Class caller = null;
        java.lang.SecurityManager sm = java.lang.System.getSecurityManager();
        if (sm != null) {
            // Reflective call to get caller class is only needed if a security
            // manager
            // is present. Avoid the overhead of making this call otherwise.
            caller = Reflection.getCallerClass();
            if (loader == null) {
                ClassLoader ccl = ClassLoaderPeer.getClassLoader(caller);
                if (ccl != null) {
                    sm.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
                }
            }
        }
        return forName0(name, initialize, loader, caller);
    }

    /**
     * Called after security check for system loader access checks have been
     * made.
     */
    private static java.lang.Class forName0(String name, boolean initialize, java.lang.ClassLoader loader,
            java.lang.Class caller) throws ClassNotFoundException {
        java.lang.Class c = java.lang.Class.forName(Jail.classNameAddJail(name), initialize, loader); 
        Log.log(Log.LOG_REFLECT,"Class.forName ",name," -> ",(c==null)?"null":c.getCanonicalName());
        return c;
    }

    /**
     * Returns the class loader for the class. Some implementations may use null
     * to represent the bootstrap class loader. This method will return null in
     * such implementations if this class was loaded by the bootstrap class
     * loader.
     *
     * <p>
     * If a security manager is present, and the caller's class loader is not
     * null and the caller's class loader is not the same as or an ancestor of
     * the class loader for the class whose class loader is requested, then this
     * method calls the security manager's {@code checkPermission} method with a
     * {@code RuntimePermission("getClassLoader")} permission to ensure it's ok
     * to access the class loader for the class.
     *
     * <p>
     * If this object represents a primitive type or void, null is returned.
     *
     * @return the class loader that loaded the class or interface represented
     *         by this object.
     * @throws SecurityException
     *             if a security manager exists and its {@code checkPermission}
     *             method denies access to the class loader for the class.
     * @see java.lang.ClassLoader
     * @see SecurityManager#checkPermission
     * @see java.lang.RuntimePermission
     */
    public static jail.java.lang.ClassLoader getClassLoader(Object obj) {
        ClassLoader cl = ((java.lang.Class) obj).getClassLoader();
        if (cl == null)
            return null;
        jail.java.lang.ClassLoader jcl = ClassLoaderPeer.toJail(cl);
        java.lang.SecurityManager sm = java.lang.System.getSecurityManager();
        if (sm != null) {
            ClassLoaderPeer.access.checkClassLoaderPermission(jcl, Reflection.getCallerClass());
        }
        return jcl;
    }

}
